use std::os::unix::prelude::RawFd;

use anyhow::Result;
use nix::sys::termios::Termios;

/// Wraps a RawFd in order to enable raw mode.
pub struct RawTerm(RawFd, Termios);

impl RawTerm {
  /// Put the terminal into raw mode.
  ///
  /// I have no idea why these work yet, these are just copied from Python's
  /// pty.py library
  pub fn init(fd: RawFd) -> Result<Self> {
    use nix::sys::termios::*;
    let saved_mode = tcgetattr(fd)?;
    let mut mode = saved_mode.clone();
    mode.input_flags &= !(InputFlags::BRKINT
      | InputFlags::ICRNL
      | InputFlags::INPCK
      | InputFlags::ISTRIP
      | InputFlags::IXON);
    mode.output_flags &= !OutputFlags::OPOST;
    mode.control_flags &= !(ControlFlags::CSIZE | ControlFlags::PARENB);
    mode.control_flags |= ControlFlags::CS8;
    mode.local_flags &= !(LocalFlags::ECHO
      | LocalFlags::ICANON
      | LocalFlags::IEXTEN
      | LocalFlags::ISIG);
    mode.control_chars[libc::VMIN] = 1;
    mode.control_chars[libc::VTIME] = 0;
    tcsetattr(fd, SetArg::TCSAFLUSH, &mode)?;
    Ok(RawTerm(fd, saved_mode))
  }
}

impl Drop for RawTerm {
  fn drop(&mut self) {
    use nix::sys::termios::*;
    tcsetattr(self.0, SetArg::TCSAFLUSH, &self.1).unwrap();
  }
}
