//! Setting up a recorder

use std::collections::HashMap;
use std::fs::File;
use std::io::Write;
use std::path::PathBuf;
use std::process::Command;

use std::thread;

use anyhow::Result;

use crate::asciicast::HeaderBuilder;

use super::terminal::Terminal;

#[derive(Debug, Parser)]
pub struct RecordOpts {
  #[clap(long = "output", default_value = "output.cast")]
  output_file: PathBuf,
}

pub fn record_main(opts: RecordOpts) -> Result<()> {
  let mut file = File::create(&opts.output_file)?;

  let mut command = Command::new("zsh");
  command.env("TERM", "xterm-256color");

  // Write header
  // TODO: Clean this up
  let header = {
    let command_str = format!("{command:?}");
    let env = command
      .get_envs()
      .into_iter()
      .filter_map(|(a, b)| b.map(|b| (a, b)))
      .map(|(a, b)| {
        (
          a.to_string_lossy().to_string(),
          b.to_string_lossy().to_string(),
        )
      })
      .collect::<HashMap<_, _>>();

    HeaderBuilder::default()
      .width(30)
      .height(30)
      .command(command_str)
      .env(env)
      .build()?
  };

  serde_json::to_writer(&file, &header)?;
  file.write(b"\n")?;

  let (pty, rx) = Terminal::setup(command)?;
  thread::spawn(move || {
    for event in rx.into_iter() {
      serde_json::to_writer(&file, &event);
      file.write(b"\n");
    }
  });

  pty.wait_until_complete()?;

  Ok(())
}
