use std::fs::File;
use std::io::{self, Write};
use std::sync::Arc;

use parking_lot::Mutex;

use crate::asciicast::{Event, EventKind};
use crate::errors::Result;

pub trait Writer: Clone {
  fn write_line(&mut self, line: String) -> Result<(), io::Error>;

  fn write_stdin(&mut self, time: f64, data: &[u8]) -> Result<()> {
    let output = EventKind::Input(data);
    let event = Event(time, output);
    self.write_line(serde_json::to_string(&event)?)?;
    Ok(())
  }

  fn write_stdout(&mut self, time: f64, data: &[u8]) -> Result<()> {
    let output = EventKind::Output(data);
    let event = Event(time, output);
    self.write_line(serde_json::to_string(&event)?)?;
    Ok(())
  }
}

#[derive(Clone)]
pub struct FileWriter(Arc<Mutex<File>>);

impl FileWriter {
  pub fn new(file: File) -> Self {
    FileWriter(Arc::new(Mutex::new(file)))
  }
}

impl Writer for FileWriter {
  fn write_line(&mut self, line: String) -> Result<(), io::Error> {
    let mut writer = self.0.lock();
    writer.write_all(line.as_bytes())?;
    writer.write(b"\n")?;
    writer.flush()?;
    Ok(())
  }
}
